/*
 * Narratives for Power BI extension. This file is loaded by the Power BI
 * framework and executed.
 *
 * During development:
 *
 *   1. Go to https://app.powerbi.com/devTools
 *   2. Paste the contents of this file into the TypeScript pane (top left)
 *   3. Click Compile + Run and see the extension load in the preview pane on
 *      the right. Use Chrome DevTools to inspect elements, set breakpoints,
 *      etc.
 */

module powerbi.visuals {

    export interface INarrativeConfig {
        apiUrl: string;
        apiKey: string;
        betaCode: string;
        enableLogger: boolean;
        extensionVersion: string;
        mixpanelToken: string;
        releaseVersion: string;
        sentryPublicDsn: string;
        staticUrl: string;
        supportUrl: string;
        useCdn: boolean;
    }

    // Modeled after the Orientation module in https://github.com/Microsoft/PowerBI-visuals/blob/master/src/Clients/CustomVisuals/visuals/bulletChart/bulletChart.ts
    module Verbosity {
        export var type: IEnumType = powerbi.createEnumType([
            { value: 'low', displayName: 'Low' },
            { value: 'medium', displayName: 'Medium' },
            { value: 'high', displayName: 'High' }
        ]);
    }

    module Structure {
        export var type: IEnumType = powerbi.createEnumType([
            { value: 'paragraph', displayName: 'Paragraph' },
            { value: 'bullets', displayName: 'Bullet List' }
        ]);
    }

    module ValueType {
        export var type: IEnumType = powerbi.createEnumType([
            { value: 'discrete', displayName: 'Discrete' },
            { value: 'continuous', displayName: 'Continuous' },
            { value: 'percentOfWhole', displayName: 'Percent of Whole' },
            { value: 'scatterplot', displayName: 'Scatterplot' }
        ]);
    }

    export class Narrative implements IVisual {

        // TODO: add actual formatting options like verbosity, paragraph/
        // bullets as capabilities.
        public static capabilities: VisualCapabilities = {
            // This is what will appear in the 'Field Wells' in reports
            dataRoles: [
                {
                    name: 'Dimensions',
                    displayName: 'Dimensions',
                    kind: powerbi.VisualDataRoleKind.Grouping
                },
                {
                    name: 'Values',
                    displayName: 'Values',
                    kind: powerbi.VisualDataRoleKind.Measure
                },
            ],
            // This tells power bi how to map your roles above into the
            // dataview you will receive
            dataViewMappings: [{
                categorical: {
                    categories: {
                        for: { in: 'Dimensions' },
                        dataReductionAlgorithm: { top: {} }
                    },
                    values: {
                        select: [{ bind: { to: 'Values' } }]
                    }
                }
            }],
            // Objects light up the formatting pane
            objects: {
                Narrative: {
                    displayName: 'Narrative',
                    properties: {
                        Type: {
                            displayName: 'Type',
                            type: { enumeration: ValueType.type }
                        },
                        Structure: {
                            displayName: 'Structure',
                            type: { enumeration: Structure.type }
                        },
                        Verbosity: {
                            displayName: 'Verbosity',
                            type: { enumeration: Verbosity.type }
                        },
                        'Font Size': {
                            displayName: 'Font Size',
                            type: { formatting: { fontSize: true } }
                        }
                    }
                },
                // The hidden properties will not be shown (i.e. they won't be
                // enumerated in enumerateObjectInstances)
                hidden: {
                    properties: {
                        settings: {
                            type: { text: true }
                        }
                    }
                }
            }
        };

        private static config: INarrativeConfig = {
            apiKey: '7d6453f8d5eb8e75ee008bc091e0b2b9',
            apiUrl: 'https://prod-viz-saas-api.narrativescience.com',
            betaCode: '48f28bd7-2101-44be-b58f-04f9608dfe4b',
            enableLogger: parseBool('false'),
            extensionVersion: '0.3.20',
            mixpanelToken: 'b48ef09750a3a66c7965b94c1403cc16',
            releaseVersion: '0.0',
            sentryPublicDsn: 'https://871d577cf2894996909c31577343fddd@app.getsentry.com/69551',
            staticUrl: 'https://prod-viz-saas-extensions.narrativescience.com/v1/extensions/powerbi/0.0/static',
            supportUrl: 'https://powerbi-support.narrativescience.com',
            useCdn: parseBool('true')
        }

        private dataView: DataView;
        private narrative;
        private lastUpdateOptions;

        // This is called once when the visual is initialially created. It
        // will asynchronously load the extension from our landing page server
        // then instantiate a narrative instance once the script has executed.
        public init(options: VisualInitOptions): void {
            if (!window.nbi) {
                $.ajaxSetup({
                    cache: true
                });
                var url = Narrative.config.staticUrl + '/main.js' +
                  '?user_key=' + Narrative.config.apiKey +
                  '&code=' + Narrative.config.betaCode;
                $.getScript(url)
                    .done((params) => {
                        this.narrative = window.nbi.initialize(
                            Narrative.config, options);
                        this.narrative.update(this.lastUpdateOptions);
                        this.lastUpdateOptions = null;
                    });
            } else {
                this.narrative = window.nbi.initialize(
                    Narrative.config, options);
            }
        }

        // Update is called for data updates, resizes & formatting changes.
        // Wait until the narrative instance is created before calling its
        // update method.
        public update(options: VisualUpdateOptions) {
            if (!options.dataViews) {
                return;
            }
            this.dataView = options.dataViews[0];
            if (!this.narrative) {
                this.lastUpdateOptions = options;
            } else {
                this.narrative.update(options);
            }
        }

        // Method is called when rendering the Power BI formatting pane. It
        // needs to return an array of formatting objects that will be
        // displayed. This function should also take care of handling missing
        // or invalid values. See the capabilities.objects object above.
        public enumerateObjectInstances(options: EnumerateVisualObjectInstancesOptions): VisualObjectInstance[] {
            var instances: VisualObjectInstance[] = [];
            var dataView = this.dataView;
            switch (options.objectName) {
                case 'Narrative':
                    var general: VisualObjectInstance = {
                        objectName: 'Narrative',
                        displayName: 'Narrative',
                        selector: null,
                        properties: {
                            Type: Narrative.getValueType(dataView),
                            Structure: Narrative.getStructure(dataView),
                            Verbosity: Narrative.getVerbosity(dataView),
                            'Font Size': Narrative.getFontSize(dataView)
                        }
                    };
                    instances.push(general);
                    break;
            }
            return instances;
        }

        // Method will be called when the view mode changes. Interpret the
        // view mode change in the narrative instance.
        public onViewModeChanged(viewMode: ViewMode): void {
            if (!this.narrative) {
                return;
            }
            this.narrative.setViewMode(viewMode);
        }

        // Get the valueType value from the data view. Defaults to discrete.
        private static getValueType(dataView: DataView): string {
            var valueType = <string>this.getFormatValue(dataView, 'Type')
            if (valueType && ~['discrete', 'continuous', 'percentOfWhole', 'scatterplot'].indexOf(valueType)) {
                return valueType;
            }
            return 'discrete';
        }

        // Get the structure value from the data view. Defaults to paragraph.
        private static getStructure(dataView: DataView): string {
            var structure = <string>this.getFormatValue(dataView, 'Structure')
            if (structure && ~['paragraph', 'bullets'].indexOf(structure)) {
                return structure;
            }
            return 'paragraph';
        }

        // Get the verbosity value from the data view. Defaults to 5.
        private static getVerbosity(dataView: DataView): number {
            var verbosity = <string>this.getFormatValue(dataView, 'Verbosity')
            if (~['low', 'medium', 'high'].indexOf(verbosity)) {
                return verbosity;
            }
            return 'medium';
        }

        // Get the font size value from the data view. Defaults to 14.
        private static getFontSize(dataView: DataView): number {
            var size = <number>this.getFormatValue(dataView, 'Font Size')
            if (size) {
                return size;
            }
            return 14;
        }

        // Returns the format value for the given key found in narrative
        // object. This handles missing objects and returns null if nothing
        // was found.
        private static getFormatValue(dataView: DataView, key: string): any {
            if (dataView) {
                var objects = dataView.metadata.objects;
                if (objects) {
                    var general = objects['Narrative'];
                    if (general) {
                        var value = general[key];
                        if (value != null)
                            return value;
                    }
                }
            }
            return null;
        }

        public destroy() {
          this.narrative.destroy()
        }

    }
}

function parseBool(str: string): boolean {
    var strLower = str.toLowerCase();
    if (strLower === 'true' ||
            strLower === '1' ||
            strLower === 'yes') {
        return true;
    }
    return false;
}
